mod commands;
mod util;

use serenity::all::OnlineStatus;
use serenity::async_trait;
use serenity::builder::{CreateInteractionResponse, CreateInteractionResponseMessage};
use serenity::gateway::ActivityData;
use serenity::model::application::{Command, Interaction};
use serenity::model::gateway::Ready;
use serenity::prelude::*;
use util::config;

struct Handler;

#[async_trait]
impl EventHandler for Handler {
    async fn interaction_create(&self, ctx: Context, interaction: Interaction) {
        if let Interaction::Command(command) = interaction {
            println!("Received command interaction: {command:#?}");

            let content = match command.data.name.as_str() {
                "ping" => Some(commands::ping::run(&command.data.options())),
                _ => Some("not implemented :(".to_string()),
            };

            if let Some(content) = content {
                let data = CreateInteractionResponseMessage::new().content(content);
                let builder = CreateInteractionResponse::Message(data);
                if let Err(why) = command.create_response(&ctx.http, builder).await {
                    println!("Cannot respond to slash command: {why}");
                }
            }
        }
    }

    async fn ready(&self, ctx: Context, ready: Ready) {
        println!("{} is connected!", ready.user.name);

        let _guild_command =
            Command::create_global_command(&ctx.http, commands::ping::register())
                .await;

        println!("Created all public / commands");
    }
}

#[tokio::main]
async fn main() {
    println!(r"__________               __                      .___      
\______   \__ __ _______/  |_   ____    ____   __| _/____  
 |       _/  |  |  ___/\   __\_/ __ \  /    \ / __ |/  _ \ 
 |    |   \  |  |___ \  |  |  \  ___/ |   |  | /_/ (  <_> )
 |____|_  /____/____  > |__|   \___  >|___|  |____ |\____/ 
        \/          \/             \/      \/     \/       
");



    let config = config::load().unwrap();

    let status = OnlineStatus::DoNotDisturb;
    let activity = ActivityData::streaming("music", "https://twitch.tv/moonleaytv").unwrap();
    // Build our client.
    let mut client = Client::builder(config.discord_token, GatewayIntents::empty())
        .event_handler(Handler)
        .status(status)
        .activity(activity)
        .await
        .expect("Error creating client");

    // Finally, start a single shard, and start listening to events.
    //
    // Shards will automatically attempt to reconnect, and will perform exponential backoff until
    // it reconnects.
    if let Err(why) = client.start().await {
        println!("Client error: {why:?}");
    }
}
